@@ -31,6 +31,8 @@ def subscription_to_identifier(subscription: Subscription) -> str:
         return f'userNonFundingLedgerUpdates:{subscription["user"].lower()}'
     elif subscription["type"] == "webData2":
         return f'webData2:{subscription["user"].lower()}'
+    elif subscription["type"] == "bbo":
+        return f'bbo:{subscription["coin"].lower()}'
 
 
 def ws_msg_to_identifier(ws_msg: WsMsg) -> Optional[str]:
@@ -60,6 +62,8 @@ def ws_msg_to_identifier(ws_msg: WsMsg) -> Optional[str]:
         return f'userNonFundingLedgerUpdates:{ws_msg["data"]["user"].lower()}'
     elif ws_msg["channel"] == "webData2":
         return f'webData2:{ws_msg["data"]["user"].lower()}'
+    elif ws_msg["channel"] == "bbo":
+        return f'bbo:{ws_msg["data"]["coin"].lower()}'
 
 
 class WebsocketManager(threading.Thread):
